@@ -2,7 +2,7 @@
 #
 #  Net::Server::MultiType - Net::Server personality
 #
-#  $Id: MultiType.pm,v 1.15 2012/05/30 13:46:11 rhandom Exp $
+#  $Id: MultiType.pm,v 1.16 2012/06/05 22:43:05 rhandom Exp $
 #
 #  Copyright (C) 2001-2012
 #
@@ -33,6 +33,7 @@ sub options {
     return $ref;
 }
 
+sub default_server_type { 'Fork' }
 
 sub run {
     my $self = ref($_[0]) ? shift() : shift->new;
@@ -57,6 +58,19 @@ sub run {
             next;
         }
 
+        # handle items like HTTP and PSGI that aren't true Net::Server flavors, but themselves are MultiType
+        if ($pkg->isa(__PACKAGE__)) {
+            my $type = $self->default_server_type || 'Single';
+            $type = ($type =~ /^(\w+)$/) ? $1 : next; # satisfy taint
+            my $_pkg = ($type =~ /::/) ? $type : "Net::Server::$type";
+            $prop->{'_recursive_multitype'} = $_pkg;
+            (my $file = "$_pkg.pm") =~ s{::}{/}g;
+            eval { require $file } or die "Trouble becoming server type $pkg while loading default package $_pkg: $@\n";
+            die "Recursive inheritance - Package $pkg inherits from $_pkg.\n" if $_pkg->isa($pkg);
+            no strict 'refs';
+            @{"${pkg}::ISA"} = ($_pkg);
+        }
+
         # cludgy - doesn't allow multiple Net::Server::MultiType servers within same process
         # but it is probably better than modifying our child's class for it
         @Net::Server::MultiType::ISA = ($pkg);
